@@ -147,7 +147,9 @@
 #include "dragonfly.h"
 #elif defined(__OpenBSD__)
 #include "openbsd.h"
-#endif /* __OpenBSD__ */
+#elif defined(__NetBSD__)
+#include "netbsd.h"
+#endif
 
 #include "gradient.h"
 
@@ -2061,7 +2063,7 @@ void load_config_file() {
 
 inline void reset_optind() {
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
-    defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
+    defined(__OpenBSD__) || defined(__DragonFly__)
   optind = optreset = 1;
 #else
   optind = 0;
@@ -2107,6 +2109,10 @@ void set_current_config() {
 /* : means that character before that takes an argument */
 const char *getopt_string =
     "vVqdDSs:t:u:i:hc:p:"
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
+    defined(__HAIKU__)
+    "U"
+#endif /* Linux || FreeBSD || Haiku */
 #ifdef BUILD_X11
     "x:y:w:a:X:m:f:"
 #ifdef OWN_WINDOW
@@ -2136,7 +2142,13 @@ const struct option longopts[] = {
     {"double-buffer", 0, nullptr, 'b'}, {"window-id", 1, nullptr, 'w'},
 #endif /* BUILD_X11 */
     {"text", 1, nullptr, 't'},          {"interval", 1, nullptr, 'u'},
-    {"pause", 1, nullptr, 'p'},         {nullptr, 0, nullptr, 0}};
+    {"pause", 1, nullptr, 'p'},
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
+    defined(__HAIKU__)
+    {"unique", 0, nullptr, 'U'},
+#endif /* Linux || FreeBSDi || Haiku */
+    {nullptr, 0, nullptr, 0}
+};
 
 void setup_inotify() {
 #ifdef HAVE_SYS_INOTIFY_H
