@@ -44,10 +44,18 @@
 #include "ccurl_thread.h"
 #endif /* BUILD_CURL */
 
+#if defined(__linux__)
+#include "linux.h"
+#endif /* Linux */
+
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
 #include "freebsd.h"
 #endif /* FreeBSD */
 
+#if defined(__HAIKU__)
+#include "haiku.h"
+#endif /* Haiku */
+
 #ifdef BUILD_BUILTIN_CONFIG
 #include "defconfig.h"
 
@@ -120,9 +128,9 @@ static void print_version() {
 #ifdef BUILD_IMLIB2
             << _("  * Imlib2\n")
 #endif /* BUILD_IMLIB2 */
-#ifdef HAVE_SOME_SOUNDCARD_H
+#ifdef HAVE_SOUNDCARD_H
             << _("  * OSS mixer support\n")
-#endif /* HAVE_SOME_SOUNDCARD_H */
+#endif /* HAVE_SOUNDCARD_H */
 #ifdef BUILD_MIXER_ALSA
             << _("  * ALSA mixer support\n")
 #endif /* BUILD_MIXER_ALSA */
@@ -268,13 +276,17 @@ static void print_help(const char *prog_name) {
          "   -i COUNT                  number of times to update " PACKAGE_NAME
          " (and quit)\n"
          "   -p, --pause=SECS          pause for SECS seconds at startup "
-         "before doing anything\n",
-         prog_name);
+         "before doing anything\n"
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
+    defined(__HAIKU__)
+         "   -U, --unique              only one conky process can be created\n"
+#endif /* Linux || FreeBSD || Haiku */
+         , prog_name);
 }
 
 inline void reset_optind() {
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
-    defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
+    defined(__OpenBSD__) || defined(__DragonFly__)
   optind = optreset = 1;
 #else
   optind = 0;
@@ -293,6 +305,8 @@ int main(int argc, char **argv) {
   g_sighup_pending = 0;
   g_sigusr2_pending = 0;
 
+  bool unique_process = false;
+
 #ifdef BUILD_CURL
   struct curl_global_initializer {
     curl_global_initializer() {
@@ -349,12 +363,25 @@ int main(int argc, char **argv) {
         window.window = strtol(optarg, nullptr, 0);
         break;
 #endif /* BUILD_X11 */
-
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
+    defined(__HAIKU__)
+      case 'U':
+        unique_process = true;
+        break;
+#endif /* Linux || FreeBSD || Haiku */
       case '?':
         return EXIT_FAILURE;
     }
   }
 
+#if defined(__linux__) || defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || \
+    defined(__HAIKU__)
+  if (unique_process && is_conky_already_running()) {
+    NORM_ERR("already running");
+    return 0;
+  }
+#endif /* Linux || FreeBSD || Haiku */
+
   try {
     set_current_config();
 
